@@ -1,6 +1,6 @@
 /*
  * JasperReports - Free Java Reporting Library.
- * Copyright (C) 2001 - 2009 Jaspersoft Corporation. All rights reserved.
+ * Copyright (C) 2001 - 2011 Jaspersoft Corporation. All rights reserved.
  * http://www.jaspersoft.com
  *
  * Unless you have purchased a commercial license agreement from Jaspersoft,
@@ -17,7 +17,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public License
  * along with JasperReports. If not, see <http://www.gnu.org/licenses/>.
  */
@@ -42,6 +42,7 @@ import java.io.OutputStreamWriter;
 import java.io.StringWriter;
 import java.io.Writer;
 import java.text.AttributedCharacterIterator;
+import java.text.AttributedCharacterIterator.Attribute;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
@@ -68,8 +69,9 @@ import net.sf.jasperreports.engine.JRPrintRectangle;
 import net.sf.jasperreports.engine.JRPrintText;
 import net.sf.jasperreports.engine.JRRenderable;
 import net.sf.jasperreports.engine.JRWrappingSvgRenderer;
-import net.sf.jasperreports.engine.JasperPrint;
+import net.sf.jasperreports.engine.TabStop;
 import net.sf.jasperreports.engine.base.JRBaseFont;
+import net.sf.jasperreports.engine.base.JRBasePrintText;
 import net.sf.jasperreports.engine.fonts.FontFamily;
 import net.sf.jasperreports.engine.fonts.FontInfo;
 import net.sf.jasperreports.engine.type.LineDirectionEnum;
@@ -89,13 +91,15 @@ import org.apache.commons.logging.LogFactory;
  * a free-form layout. It uses the RTF Specification 1.6 (compatible with MS Word 6.0, 2003 and XP).
  * 
  * @author Flavius Sana (flavius_sana@users.sourceforge.net)
- * @version $Id: JRRtfExporter.java 3859 2010-06-03 13:43:44Z lucianc $
+ * @version $Id: JRRtfExporter.java 4595 2011-09-08 15:55:10Z teodord $
  */
 public class JRRtfExporter extends JRAbstractExporter
 {
 	private static final Log log = LogFactory.getLog(JRRtfExporter.class);
 	
 	private static final String RTF_EXPORTER_PROPERTIES_PREFIX = JRProperties.PROPERTY_PREFIX + "export.rtf.";
+	
+	private static final int LINE_SPACING_FACTOR = 240; //(int)(240 * 2/3f);
 
 	/**
 	 * The exporter key, as used in
@@ -107,18 +111,18 @@ public class JRRtfExporter extends JRAbstractExporter
 	 *
 	 */
 	protected static final String JR_PAGE_ANCHOR_PREFIX = "JR_PAGE_ANCHOR_";
-	protected JRExportProgressMonitor progressMonitor = null;
+	protected JRExportProgressMonitor progressMonitor;
 
-	protected FileBufferedWriter colorWriter = null;
-	protected FileBufferedWriter fontWriter = null;
-	protected FileBufferedWriter writer = null;
-	protected Writer rtfWriter = null;
-	protected File destFile = null;
+	protected FileBufferedWriter colorWriter;
+	protected FileBufferedWriter fontWriter;
+	protected FileBufferedWriter writer;
+	protected Writer rtfWriter;
+	protected File destFile;
 
-	protected int reportIndex = 0;
+	protected int reportIndex;
 
-	protected List colors = null;
-	protected List fonts = null;
+	protected List<Color> colors;
+	protected List<String> fonts;
 
 	// z order of the graphical objects in .rtf file
 	private int zorder = 1;
@@ -126,7 +130,7 @@ public class JRRtfExporter extends JRAbstractExporter
 	/**
 	 * @deprecated
 	 */
-	private Map fontMap = null;
+	private Map<String,String> fontMap;
 
 	protected class ExporterContext extends BaseExporterContext implements JRRtfExporterContext
 	{
@@ -166,11 +170,11 @@ public class JRRtfExporter extends JRAbstractExporter
 				setPageRange();
 			}
 
-			fonts = new ArrayList();
-			colors = new ArrayList();
+			fonts = new ArrayList<String>();
+			colors = new ArrayList<Color>();
 			colors.add(null);
 
-			fontMap = (Map) parameters.get(JRExporterParameter.FONT_MAP);
+			fontMap = (Map<String,String>) parameters.get(JRExporterParameter.FONT_MAP);
 			setHyperlinkProducerFactory();
 			StringBuffer sb = (StringBuffer)parameters.get(JRExporterParameter.OUTPUT_STRING_BUFFER);
 			if (sb != null) {
@@ -257,9 +261,9 @@ public class JRRtfExporter extends JRAbstractExporter
 		writer = new FileBufferedWriter();
 
 		for(reportIndex = 0; reportIndex < jasperPrintList.size(); reportIndex++ ){
-			setJasperPrint((JasperPrint)jasperPrintList.get(reportIndex));
+			setJasperPrint(jasperPrintList.get(reportIndex));
 
-			List pages = jasperPrint.getPages();
+			List<JRPrintPage> pages = jasperPrint.getPages();
 			if (pages != null && pages.size() > 0){
 				if (isModeBatch)
 				{
@@ -273,18 +277,20 @@ public class JRRtfExporter extends JRAbstractExporter
 				writer.write("}}\n");
 
 				writer.write("\\viewkind1\\paperw");
-				writer.write(String.valueOf(twip(jasperPrint.getPageWidth())));
+				writer.write(String.valueOf(LengthUtil.twip(jasperPrint.getPageWidth())));
 				writer.write("\\paperh");
-				writer.write(String.valueOf(twip(jasperPrint.getPageHeight())));
+				writer.write(String.valueOf(LengthUtil.twip(jasperPrint.getPageHeight())));
 
 				writer.write("\\marglsxn");
-				writer.write(String.valueOf(twip(jasperPrint.getLeftMargin() == null ? 0 : jasperPrint.getLeftMargin())));
+				writer.write(String.valueOf(LengthUtil.twip(jasperPrint.getLeftMargin() == null ? 0 : jasperPrint.getLeftMargin())));
 				writer.write("\\margrsxn");
-				writer.write(String.valueOf(twip(jasperPrint.getRightMargin() == null ? 0 : jasperPrint.getRightMargin())));
+				writer.write(String.valueOf(LengthUtil.twip(jasperPrint.getRightMargin() == null ? 0 : jasperPrint.getRightMargin())));
 				writer.write("\\margtsxn");
-				writer.write(String.valueOf(twip(jasperPrint.getTopMargin() == null ? 0 : jasperPrint.getTopMargin())));
+				writer.write(String.valueOf(LengthUtil.twip(jasperPrint.getTopMargin() == null ? 0 : jasperPrint.getTopMargin())));
 				writer.write("\\margbsxn");
-				writer.write(String.valueOf(twip(jasperPrint.getBottomMargin() == null ? 0 : jasperPrint.getBottomMargin())));
+				writer.write(String.valueOf(LengthUtil.twip(jasperPrint.getBottomMargin() == null ? 0 : jasperPrint.getBottomMargin())));
+				writer.write("\\deftab");
+				writer.write(String.valueOf(LengthUtil.twip(new JRBasePrintText(jasperPrint.getDefaultStyleProvider()).getParagraph().getTabStopWidth())));
 
 				if (jasperPrint.getOrientationValue() == OrientationEnum.LANDSCAPE) {
 					writer.write("\\lndscpsxn");
@@ -297,7 +303,7 @@ public class JRRtfExporter extends JRAbstractExporter
 						throw new JRException("Current thread interrupted");
 					}
 
-					page = (JRPrintPage)pages.get(pageIndex);
+					page = pages.get(pageIndex);
 					writeAnchor(JR_PAGE_ANCHOR_PREFIX + reportIndex + "_" + (pageIndex + 1));
 
 					boolean lastPageFlag = false;
@@ -397,7 +403,7 @@ public class JRRtfExporter extends JRAbstractExporter
 		String fontName = font.getFontName();
 		if(fontMap != null && fontMap.containsKey(fontName))
 		{
-			fontName = (String)fontMap.get(fontName);
+			fontName = fontMap.get(fontName);
 		}
 		else
 		{
@@ -425,25 +431,6 @@ public class JRRtfExporter extends JRAbstractExporter
 		return fontIndex;
 	}
 
-	/**
-	 * Convert a int value from points to EMU (multiply with 12700)
-	 * @param points value that needs to be converted
-	 * @return converted value in EMU
-	 */
-	private int emu(float points) {
-		return (int)(points * 12700);
-	}
-
-
-	/**
-	 * Convert a float value to twips (multiply with 20)
-	 * @param points value that need to be converted
-	 * @return converted value in twips
-	 */
-	private int twip(float points) {
-		return (int)(points * 20);
-	}
-
 
 	/**
 	 * Exports a report page
@@ -468,13 +455,13 @@ public class JRRtfExporter extends JRAbstractExporter
 		writer.write("{\\shp\\shpbxpage\\shpbypage\\shpwr5\\shpfhdr0\\shpfblwtxt0\\shpz");
 		writer.write(String.valueOf(zorder++));
 		writer.write("\\shpleft");
-		writer.write(String.valueOf(twip(element.getX() + getOffsetX())));
+		writer.write(String.valueOf(LengthUtil.twip(element.getX() + getOffsetX())));
 		writer.write("\\shpright");
-		writer.write(String.valueOf(twip(element.getX() + getOffsetX() + element.getWidth())));
+		writer.write(String.valueOf(LengthUtil.twip(element.getX() + getOffsetX() + element.getWidth())));
 		writer.write("\\shptop");
-		writer.write(String.valueOf(twip(element.getY() + getOffsetY())));
+		writer.write(String.valueOf(LengthUtil.twip(element.getY() + getOffsetY())));
 		writer.write("\\shpbottom");
-		writer.write(String.valueOf(twip(element.getY() + getOffsetY() + element.getHeight())));
+		writer.write(String.valueOf(LengthUtil.twip(element.getY() + getOffsetY() + element.getHeight())));
 
 		Color bgcolor = element.getBackcolor();
 
@@ -545,7 +532,7 @@ public class JRRtfExporter extends JRAbstractExporter
 		}
 
 		writer.write("{\\sp{\\sn lineWidth}{\\sv ");
-		writer.write(String.valueOf(emu(lineWidth)));
+		writer.write(String.valueOf(LengthUtil.emu(lineWidth)));
 		writer.write("}}");
 	}
 
@@ -590,13 +577,13 @@ public class JRRtfExporter extends JRAbstractExporter
 		writer.write("{\\shp\\shpbxpage\\shpbypage\\shpwr5\\shpfhdr0\\shpz");
 		writer.write(String.valueOf(zorder++));
 		writer.write("\\shpleft");
-		writer.write(String.valueOf(twip(x)));
+		writer.write(String.valueOf(LengthUtil.twip(x)));
 		writer.write("\\shpright");
-		writer.write(String.valueOf(twip(x + width)));
+		writer.write(String.valueOf(LengthUtil.twip(x + width)));
 		writer.write("\\shptop");
-		writer.write(String.valueOf(twip(y)));
+		writer.write(String.valueOf(LengthUtil.twip(y)));
 		writer.write("\\shpbottom");
-		writer.write(String.valueOf(twip(y + height)));
+		writer.write(String.valueOf(LengthUtil.twip(y + height)));
 
 		writer.write("{\\shpinst");
 		
@@ -625,13 +612,13 @@ public class JRRtfExporter extends JRAbstractExporter
 		writer.write("{\\shp\\shpbxpage\\shpbypage\\shpwr5\\shpfhdr0\\shpz");
 		writer.write(String.valueOf(zorder++));
 		writer.write("\\shpleft");
-		writer.write(String.valueOf(twip(x)));//FIXMEBORDER starting point of borders seem to have CAP_SQUARE-like appearence at least for Thin
+		writer.write(String.valueOf(LengthUtil.twip(x)));//FIXMEBORDER starting point of borders seem to have CAP_SQUARE-like appearence at least for Thin
 		writer.write("\\shpright");
-		writer.write(String.valueOf(twip(x + width)));
+		writer.write(String.valueOf(LengthUtil.twip(x + width)));
 		writer.write("\\shptop");
-		writer.write(String.valueOf(twip(y)));
+		writer.write(String.valueOf(LengthUtil.twip(y)));
 		writer.write("\\shpbottom");
-		writer.write(String.valueOf(twip(y + height)));
+		writer.write(String.valueOf(LengthUtil.twip(y + height)));
 
 		writer.write("{\\shpinst");
 		
@@ -814,21 +801,54 @@ public class JRRtfExporter extends JRAbstractExporter
 
 		writer.write(rotation);
 		writer.write("{\\sp{\\sn dyTextTop}{\\sv ");
-		writer.write(String.valueOf(emu(topPadding)));
+		writer.write(String.valueOf(LengthUtil.emu(topPadding)));
 		writer.write("}}");
 		writer.write("{\\sp{\\sn dxTextLeft}{\\sv ");
-		writer.write(String.valueOf(emu(leftPadding)));
+		writer.write(String.valueOf(LengthUtil.emu(leftPadding)));
 		writer.write("}}");
 		writer.write("{\\sp{\\sn dyTextBottom}{\\sv ");
-		writer.write(String.valueOf(emu(bottomPadding)));
+		writer.write(String.valueOf(LengthUtil.emu(bottomPadding)));
 		writer.write("}}");
 		writer.write("{\\sp{\\sn dxTextRight}{\\sv ");
-		writer.write(String.valueOf(emu(rightPadding)));
+		writer.write(String.valueOf(LengthUtil.emu(rightPadding)));
 		writer.write("}}");
 		writer.write("{\\sp{\\sn fLine}{\\sv 0}}");
-		writer.write("{\\shptxt{\\pard");
+		writer.write("{\\shptxt{\\pard ");
 
-		JRFont font = text;
+		writer.write("\\fi" + LengthUtil.twip(text.getParagraph().getFirstLineIndent().intValue()) + " ");
+		writer.write("\\li" + LengthUtil.twip(text.getParagraph().getLeftIndent().intValue()) + " ");
+		writer.write("\\ri" + LengthUtil.twip(text.getParagraph().getRightIndent().intValue()) + " ");
+		writer.write("\\sb" + LengthUtil.twip(text.getParagraph().getSpacingBefore().intValue()) + " ");
+		writer.write("\\sa" + LengthUtil.twip(text.getParagraph().getSpacingAfter().intValue()) + " ");
+
+		TabStop[] tabStops = text.getParagraph().getTabStops();
+		if (tabStops != null && tabStops.length > 0)
+		{
+			for (int i = 0; i < tabStops.length; i++)
+			{
+				TabStop tabStop = tabStops[i];
+
+				String tabStopAlign = "";
+				
+				switch (tabStop.getAlignment())
+				{
+					case CENTER:
+						tabStopAlign = "\\tqc";
+						break;
+					case RIGHT:
+						tabStopAlign = "\\tqr";
+						break;
+					case LEFT:
+					default:
+						tabStopAlign = "";
+						break;
+				}
+
+				writer.write(tabStopAlign + "\\tx" + LengthUtil.twip(tabStop.getPosition()) + " ");
+			}
+		}
+
+//		JRFont font = text;
 		if (text.getRunDirectionValue() == RunDirectionEnum.RTL)
 		{
 			writer.write("\\rtlch");
@@ -871,9 +891,46 @@ public class JRRtfExporter extends JRAbstractExporter
 				break;
 		}
 
-		writer.write("\\sl");
-		writer.write(String.valueOf(twip(text.getLineSpacingFactor() * font.getFontSize())));
-		writer.write(" ");
+		switch (text.getParagraph().getLineSpacing())
+		{
+			case AT_LEAST:
+			{
+				writer.write("\\sl" + LengthUtil.twip(text.getParagraph().getLineSpacingSize().floatValue()));
+				writer.write(" \\slmult0 ");
+				break;
+			}
+			case FIXED:
+			{
+				writer.write("\\sl-" + LengthUtil.twip(text.getParagraph().getLineSpacingSize().floatValue()));
+				writer.write(" \\slmult0 ");
+				break;
+			}
+			case PROPORTIONAL:
+			{
+				writer.write("\\sl" + (int)(text.getParagraph().getLineSpacingSize().floatValue() * LINE_SPACING_FACTOR));
+				writer.write(" \\slmult1 ");
+				break;
+			}
+			case DOUBLE:
+			{
+				writer.write("\\sl" + (int)(2f * LINE_SPACING_FACTOR));
+				writer.write(" \\slmult1 ");
+				break;
+			}
+			case ONE_AND_HALF:
+			{
+				writer.write("\\sl" + (int)(1.5f * LINE_SPACING_FACTOR));
+				writer.write(" \\slmult1 ");
+				break;
+			}
+			case SINGLE:
+			default:
+			{
+				writer.write("\\sl" + (int)(1f * LINE_SPACING_FACTOR));
+				writer.write(" \\slmult1 ");
+				break;
+			}
+		}
 
 		if (text.getAnchorName() != null)
 		{
@@ -893,7 +950,7 @@ public class JRRtfExporter extends JRAbstractExporter
 			)
 		{
 
-			Map styledTextAttributes = iterator.getAttributes();
+			Map<Attribute,Object> styledTextAttributes = iterator.getAttributes();
 			JRFont styleFont = new JRBaseFont(styledTextAttributes);
 			Color styleForeground = (Color) styledTextAttributes.get(TextAttribute.FOREGROUND);
 			Color styleBackground = (Color) styledTextAttributes.get(TextAttribute.BACKGROUND);
@@ -1002,7 +1059,7 @@ public class JRRtfExporter extends JRAbstractExporter
 	 * @throws JRException
 	 * @throws IOException
 	 */
-	protected void exportImage(JRPrintImage printImage) throws JRException, IOException
+	public void exportImage(JRPrintImage printImage) throws JRException, IOException
 	{
 		int leftPadding = printImage.getLineBox().getLeftPadding().intValue();
 		int topPadding = printImage.getLineBox().getTopPadding().intValue();
@@ -1165,13 +1222,13 @@ public class JRRtfExporter extends JRAbstractExporter
 			writer.write("{\\shp{\\*\\shpinst\\shpbxpage\\shpbypage\\shpwr5\\shpfhdr0\\shpfblwtxt0\\shpz");
 			writer.write(String.valueOf(zorder++));
 			writer.write("\\shpleft");
-			writer.write(String.valueOf(twip(printImage.getX() + leftPadding + xoffset + getOffsetX())));
+			writer.write(String.valueOf(LengthUtil.twip(printImage.getX() + leftPadding + xoffset + getOffsetX())));
 			writer.write("\\shpright");
-			writer.write(String.valueOf(twip(printImage.getX() + leftPadding + xoffset + getOffsetX() + imageWidth)));
+			writer.write(String.valueOf(LengthUtil.twip(printImage.getX() + leftPadding + xoffset + getOffsetX() + imageWidth)));
 			writer.write("\\shptop");
-			writer.write(String.valueOf(twip(printImage.getY() + topPadding + yoffset + getOffsetY())));
+			writer.write(String.valueOf(LengthUtil.twip(printImage.getY() + topPadding + yoffset + getOffsetY())));
 			writer.write("\\shpbottom");
-			writer.write(String.valueOf(twip(printImage.getY() + topPadding + yoffset + getOffsetY() + imageHeight)));
+			writer.write(String.valueOf(LengthUtil.twip(printImage.getY() + topPadding + yoffset + getOffsetY() + imageHeight)));
 			writer.write("{\\sp{\\sn shapeType}{\\sv 75}}");
 			writer.write("{\\sp{\\sn fFilled}{\\sv 0}}");
 			writer.write("{\\sp{\\sn fLockAspectRatio}{\\sv 0}}");
@@ -1279,10 +1336,10 @@ public class JRRtfExporter extends JRAbstractExporter
 	}
 
 
-	protected void exportElements(Collection elements) throws JRException, IOException {
+	protected void exportElements(Collection<JRPrintElement> elements) throws JRException, IOException {
 		if (elements != null && elements.size() > 0) {
-			for (Iterator it = elements.iterator(); it.hasNext();) {
-				JRPrintElement element = (JRPrintElement)it.next();
+			for (Iterator<JRPrintElement> it = elements.iterator(); it.hasNext();) {
+				JRPrintElement element = it.next();
 				if (filter == null || filter.isToExport(element)) {
 					if (element instanceof JRPrintLine) {
 						exportLine((JRPrintLine)element);
